// Filler
// fill the unused portion of the viewport with a magnified, blurred image (by default) similar to 
// effects seen in broadcasts displaying 4:3 content of 16:9 TV screens. 

// The painter function can't be used with BufferedPainter because Filler draws outside the image bounds. Caching
// is handled internally.

var Filler = new (function(){

	var painterId = 'filler';
	var fillerPainter = null;

	var isOn = false;
	this.help = function(){
		showMessage('<html>Filler<br>Fill viewport background with a magnified image.<br><li>setFillerPainter(painter)<br><li>setOverlayPainter(painter)');
	}

	addUserCommand('filler', function(){
		Filler.help();
	}, 'Help');

	var rhino = script_engine_name === 'rhino';

	// provide paint function

	var p = function(gfx, image, img, pp){

		var _super = null;
		if(rhino){
			_super_ = this;
		}else{
			_super_ = Java.super(cpainter); // get reference to 'super'
		}
	
		var bw = _super_.getCacheObject(pp); // see if image already cached

		if(bw == null){


			var iw = img.getWidth();
			var ih = img.getHeight();

			// fill in the unused portion of the display with a magnified and blurred image

			if(iw > pp.viewWidth && ih > pp.viewHeight && (image.getWidth() < pp.viewWidth || image.getHeight() < pp.viewHeight)){
				var backX = (iw / 2) - (pp.viewWidth / 2);
				var backY = (ih / 2) - (pp.viewHeight / 2);
				var subImage = img.getSubimage(parseInt(backX,10), parseInt(backY, 10), pp.viewWidth, pp.viewHeight);
				var bw = new java.awt.image.BufferedImage(pp.viewWidth, pp.viewHeight, java.awt.image.BufferedImage.TYPE_INT_ARGB); 

				// all graphics contexts must be wrapped in Gfx()
				// because of IllegalAccessWarnings from Graal and Rhino
				var bgfx = new Gfx(bw.getGraphics());
				bgfx.drawImage(subImage, 0, 0, null);

				if(fillerPainter == null){
					// create a blur filter
					var jhFilter = new com.jhlabs.image.GaussianFilter();
					jhFilter.setRadius(50);
					jhFilter.filter(bw,bw);
				}else{
					var newProps = pp.clone();
					newProps.imageX=0;
					newProps.imageY=0;
					newProps.scaledWidth = pp.viewWidth;
					newProps.scaledHeight = pp.scaledHeight;


					if(typeof fillerPainter === 'function'){

						fillerPainter(bgfx, bw, bw, newProps);
					}else{
						fillerPainter.paint(bgfx, bw, bw, newProps);
					}
				}

				bgfx.drawImage(image, Math.max(pp.imageX, 0), Math.max(pp.imageY, 0), null);
				if(overlayPainter != null){
					if(typeof overlayPainter === 'function'){
						overlayPainter(bgfx, bw, bw, pp);
					}else{
						overlayPainter.paint(bgfx, bw, bw, pp);
					}
				}
				bgfx.dispose();

				// save the image in the cache so repaint not required until image changes
				_super_.setCacheObject(bw, pp);

				gfx.drawImage(bw, 0, 0, null);
			}

		}else{
			gfx.drawImage(bw, 0, 0, null);
		}

	};


	var cpainter = null;
	if(rhino){
		cpainter = new JavaAdapter(CachingPainter, {
			paint: p	
		});
	}else{
		var CachingPainterAdapter = Java.extend(CachingPainter);
		cpainter = new CachingPainterAdapter(){
			paint: p
		};
	}

	this.setFillerPainter = function(painter){
		if (typeof painter.useCache !== "undefined") { 
			// this is a JSImagePainter. Turn off caching as filler already caches. Also,
			// the filler image will never change if the passed in painter also caches.
			painter.useCache(false); 
		}
		fillerPainter = painter;
	}

	this.on = function(){
		setPainter(cpainter, painterId);
		//setPainter(new ComboPainter(cpainter, JSI.getPainter()), painterId);
		isOn = true;
	}

	this.off = function(){
		setPainter(null, painterId);
		isOn = false;
	}

	var linkerExists = null;

	function linkerCheck(){
		if(linkerExists == null){
			linkerExists = false;
			try{	
				var p = PAINTERLINKER;
				linkerExists = true;
			}catch(e){}
		}
	}
	var painterChanged = function(p){
		linkerCheck();
		if((!linkerExists || !PAINTERLINKER.linked) && p !== painterId){
			isOn = false;
		}
	}

	registerCallback('onPainterChanged', painterChanged);


	addUserCommand("Filler", function(){
		isOn ? Filler.off() : Filler.on();

	});

	this.setOverlayPainter = function(op){
		overlayPainter = op;
	}
	var overlayPainter = null;

})();
